"""Input validation utilities for SDKRouter SDK.

Catches invalid inputs before they hit the API — saves time and money.
"""


def validate_text_input(text: str, field_name: str = "input") -> None:
    """Validate that text is suitable for LLM/embedding API calls.

    Raises:
        ValueError: If text is empty, binary, or contains NUL bytes.
    """
    if not isinstance(text, str):
        raise ValueError(f"{field_name} must be a string, got {type(text).__name__}")

    if not text.strip():
        raise ValueError(f"{field_name} is empty")

    # NUL bytes — PostgreSQL and most APIs can't handle these
    if "\x00" in text:
        raise ValueError(f"{field_name} contains NUL (\\x00) bytes — likely binary data")

    # High ratio of non-printable chars = binary content
    sample = text[:500]
    non_printable = sum(1 for c in sample if ord(c) < 32 and c not in "\n\r\t")
    if len(sample) > 20 and non_printable > len(sample) * 0.1:
        raise ValueError(
            f"{field_name} appears to contain binary data "
            f"({non_printable}/{len(sample)} non-printable chars in first 500 bytes)"
        )


def validate_embedding_input(input_data: str | list[str]) -> list[str]:
    """Validate and normalize embedding input.

    Args:
        input_data: Single string or list of strings to embed.

    Returns:
        List of validated strings.

    Raises:
        ValueError: If input is invalid.
    """
    if isinstance(input_data, str):
        texts = [input_data]
    elif isinstance(input_data, list):
        texts = input_data
    else:
        raise ValueError(f"input must be str or list[str], got {type(input_data).__name__}")

    if not texts:
        raise ValueError("input is empty — nothing to embed")

    for i, text in enumerate(texts):
        validate_text_input(text, field_name=f"input[{i}]")

    return texts


def validate_messages(messages: list) -> None:
    """Validate chat messages contain text content, not binary.

    Args:
        messages: List of message dicts with 'content' field.

    Raises:
        ValueError: If any message content is binary.
    """
    for i, msg in enumerate(messages):
        if not isinstance(msg, dict):
            continue
        content = msg.get("content")
        if isinstance(content, str) and content:
            validate_text_input(content, field_name=f"messages[{i}].content")



# Auto-generated by DjangoCFG - see CLAUDE.md

from .client import EmailGatewayEmailAPI
from .models import (
    EmailAccountCreateRequest,
    EmailAccountList,
    EmailMessageList,
    EmailSendRequest,
    EmailSendResponse,
    TestConnectionResponse,
)
from .sync_client import SyncEmailGatewayEmailAPI

__all__ = [
    "EmailGatewayEmailAPI",
    "SyncEmailGatewayEmailAPI",
    "EmailAccountCreateRequest",
    "EmailAccountList",
    "EmailMessageList",
    "EmailSendRequest",
    "EmailSendResponse",
    "TestConnectionResponse",
]


# Auto-generated by DjangoCFG - see CLAUDE.md

from __future__ import annotations
import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from ..enums import EmailMessageListStatus


class EmailMessageList(BaseModel):
    """
    
    Response model (includes read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    id: str = ...
    account_alias: Any = ...
    to_email: str = ...
    subject: Any = ...
    status: EmailMessageListStatus = Field(description='* `queued` - Queued * `sent` - S...')
    error: Any = ...
    sent_at: datetime.datetime | None = None
    created_at: datetime.datetime = ...



class EmailAccountList(BaseModel):
    """
    
    Response model (includes read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    id: str = ...
    alias: Any = Field(description="Human-readable name, e.g. 'notif...")
    smtp_host: Any = ...
    smtp_port: int = ...
    smtp_username: Any = ...
    from_email: str = ...
    from_name: Any = ...
    use_tls: bool = ...
    use_ssl: bool = ...
    is_active: bool = ...
    is_default: bool = Field(description='Default account for this user')
    total_sent: int = ...
    total_failed: int = ...
    last_sent_at: datetime.datetime | None = None
    created_at: datetime.datetime = ...



class EmailSendRequest(BaseModel):
    """
    
    Request model (no read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    account_alias: str | None = Field(
    None,
    description='Account alias. Uses default if o...',
    min_length=1,
)
    to: str = Field(min_length=1)
    cc: str | None = Field(None, min_length=1)
    bcc: str | None = Field(None, min_length=1)
    subject: str = Field(min_length=1, max_length=500)
    body: str | None = Field(None, description='Plain text body', min_length=1)
    html: str | None = Field(None, description='HTML body', min_length=1)



class EmailSendResponse(BaseModel):
    """
    
    Response model (includes read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    message_id: str = ...
    status: str = ...
    to: str = ...
    subject: str = ...



class EmailAccountCreateRequest(BaseModel):
    """
    
    Request model (no read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    alias: str = Field(min_length=1, max_length=100)
    smtp_host: str = Field(min_length=1, max_length=255)
    smtp_port: int | None = None
    smtp_username: str = Field(min_length=1, max_length=255)
    smtp_password: str = Field(min_length=1)
    from_email: str = Field(min_length=1)
    from_name: str | None = Field(None, min_length=1, max_length=200)
    use_tls: bool | None = None
    use_ssl: bool | None = None
    is_default: bool | None = None



class TestConnectionResponse(BaseModel):
    """
    
    Response model (includes read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    ok: bool = ...
    message: str = ...






"""Email gateway tool — send emails through stored SMTP accounts."""

import logging

from .._config import SDKConfig
from ..exceptions import api_error_handler, async_api_error_handler
from .._api.client import (
    BaseResource,
    AsyncBaseResource,
    SyncEmailGatewayEmailAPI,
    EmailGatewayEmailAPI,
)
from .._api.generated.email_gateway.email_gateway__api__email.models import (
    EmailAccountList,
    EmailMessageList,
    EmailSendRequest,
    EmailSendResponse,
    EmailAccountCreateRequest,
    TestConnectionResponse,
)

logger = logging.getLogger(__name__)


class EmailResource(BaseResource):
    """Email gateway (sync).

    Send emails through stored SMTP accounts via SDK API key.

    Example:
        >>> sdk = SDKRouter(api_key="sk_live_...")
        >>> sdk.email.send(to="user@example.com", subject="Hello", body="World")
        >>> sdk.email.list_accounts()
    """

    def __init__(self, config: SDKConfig):
        super().__init__(config)
        self._api = SyncEmailGatewayEmailAPI(self._http_client)

    @api_error_handler
    def send(
        self,
        to: str,
        subject: str,
        body: str = "",
        html: str = "",
        *,
        account_alias: str | None = None,
        cc: str = "",
        bcc: str = "",
    ) -> EmailSendResponse:
        """Send an email through a stored SMTP account.

        Args:
            to: Recipient email address
            subject: Email subject
            body: Plain text body
            html: HTML body (optional)
            account_alias: SMTP account alias (uses default if omitted)
            cc: Comma-separated CC addresses
            bcc: Comma-separated BCC addresses

        Returns:
            EmailSendResponse with message_id and status
        """
        data = EmailSendRequest(
            to=to,
            subject=subject,
            body=body or None,
            html=html or None,
            cc=cc or None,
            bcc=bcc or None,
            account_alias=account_alias,
        )
        return self._api.send_create(data)

    @api_error_handler
    def list_accounts(self) -> list[EmailAccountList]:
        """List configured email accounts."""
        return self._api.accounts_list()

    @api_error_handler
    def create_account(
        self,
        alias: str,
        smtp_host: str,
        smtp_username: str,
        smtp_password: str,
        from_email: str,
        *,
        smtp_port: int = 587,
        from_name: str = "",
        use_tls: bool = True,
        use_ssl: bool = False,
        is_default: bool = False,
    ) -> EmailAccountList:
        """Add a new SMTP email account."""
        return self._api.accounts_create_create(EmailAccountCreateRequest(
            alias=alias,
            smtp_host=smtp_host,
            smtp_port=smtp_port,
            smtp_username=smtp_username,
            smtp_password=smtp_password,
            from_email=from_email,
            from_name=from_name,
            use_tls=use_tls,
            use_ssl=use_ssl,
            is_default=is_default,
        ))

    @api_error_handler
    def delete_account(self, account_id: str) -> None:
        """Delete an email account."""
        self._api.accounts_destroy(account_id)

    @api_error_handler
    def test_account(self, account_id: str) -> TestConnectionResponse:
        """Test SMTP connection for an account."""
        return self._api.accounts_test_create(account_id)

    @api_error_handler
    def list_messages(self) -> list[EmailMessageList]:
        """List sent email messages."""
        return self._api.messages_list()


class AsyncEmailResource(AsyncBaseResource):
    """Email gateway (async)."""

    def __init__(self, config: SDKConfig):
        super().__init__(config)
        self._api = EmailGatewayEmailAPI(self._http_client)

    @async_api_error_handler
    async def send(
        self,
        to: str,
        subject: str,
        body: str = "",
        html: str = "",
        *,
        account_alias: str | None = None,
        cc: str = "",
        bcc: str = "",
    ) -> EmailSendResponse:
        """Send an email through a stored SMTP account."""
        data = EmailSendRequest(
            to=to, subject=subject,
            body=body or None, html=html or None,
            cc=cc or None, bcc=bcc or None,
            account_alias=account_alias,
        )
        return await self._api.send_create(data)

    @async_api_error_handler
    async def list_accounts(self) -> list[EmailAccountList]:
        return await self._api.accounts_list()

    @async_api_error_handler
    async def create_account(
        self,
        alias: str,
        smtp_host: str,
        smtp_username: str,
        smtp_password: str,
        from_email: str,
        **kwargs,
    ) -> EmailAccountList:
        return await self._api.accounts_create_create(EmailAccountCreateRequest(
            alias=alias, smtp_host=smtp_host, smtp_username=smtp_username,
            smtp_password=smtp_password, from_email=from_email, **kwargs,
        ))

    @async_api_error_handler
    async def delete_account(self, account_id: str) -> None:
        await self._api.accounts_destroy(account_id)

    @async_api_error_handler
    async def test_account(self, account_id: str) -> TestConnectionResponse:
        return await self._api.accounts_test_create(account_id)

    @async_api_error_handler
    async def list_messages(self) -> list[EmailMessageList]:
        return await self._api.messages_list()


__all__ = [
    "EmailResource",
    "AsyncEmailResource",
    "EmailAccountList",
    "EmailMessageList",
    "EmailSendRequest",
    "EmailSendResponse",
    "EmailAccountCreateRequest",
    "TestConnectionResponse",
]



"""Async SDKRouter client."""

from __future__ import annotations

import logging
from typing import Any, Optional, Type, cast

from openai import AsyncOpenAI

from .._config import SDKConfig
from .._constants import DEFAULT_MAX_RETRIES, DEFAULT_TIMEOUT
from .._types.parsed import ParsedChatCompletion
from ..utils.parsing import ResponseFormatT, type_to_response_format

from ._base import resolve_init_params
from ._helpers import LazyTool, merge_provider_extra_body, parse_completion
from ._sync import _build_parse_params

# Async tool imports
from ..tools.vision import AsyncVisionResource
from ..tools.cdn import AsyncCDNResource
from ..tools.shortlinks import AsyncShortlinksResource
from ..tools.keys import AsyncKeysResource
from ..tools.cleaner import AsyncCleanerResource
from ..tools.models import AsyncModelsResource
from ..tools.research import AsyncResearchResource
from ..tools.embeddings import AsyncEmbeddingsResource
from ..tools.image_gen import AsyncImageGenResource
from ..tools.audio import AsyncAudioResource
from ..tools.payments import AsyncPaymentsResource
from ..tools.proxies import AsyncProxiesResource
from ..tools.translator import AsyncTranslatorResource
from ..tools.knowbase import AsyncKnowledgeBaseResource
from ..tools.banner import AsyncBannerResource
from ..tools.crawler import AsyncCrawlerResource
from ..tools.email import AsyncEmailResource
from ..tools.telegram import AsyncTelegramResource

logger = logging.getLogger(__name__)


class AsyncSDKRouter(AsyncOpenAI):
    """Async SDK client — OpenAI-compatible with additional tools.

    Example:
        ```python
        from sdkrouter import AsyncSDKRouter

        client = AsyncSDKRouter(api_key="sk_live_...")
        response = await client.chat.completions.create(
            model="openai/gpt-4o",
            messages=[{"role": "user", "content": "Hello!"}],
        )
        ```
    """

    # ── Tool properties (lazy-loaded via descriptor) ─────────────────────
    vision = LazyTool("_vision", AsyncVisionResource, "Vision analysis and OCR")
    cdn = LazyTool("_cdn", AsyncCDNResource, "CDN file storage")
    shortlinks = LazyTool("_shortlinks", AsyncShortlinksResource, "URL shortening")
    keys = LazyTool("_keys", AsyncKeysResource, "API keys management")
    cleaner = LazyTool("_api_cleaner", AsyncCleanerResource, "HTML cleaner")
    llm_models = LazyTool("_models", AsyncModelsResource, "LLM models listing")
    research = LazyTool("_search", AsyncResearchResource, "Web research")
    image_gen = LazyTool("_image_gen", AsyncImageGenResource, "AI image generation")
    payments = LazyTool("_payments", AsyncPaymentsResource, "Crypto payments")
    proxies = LazyTool("_proxies", AsyncProxiesResource, "Proxy management")
    translator = LazyTool("_translator", AsyncTranslatorResource, "Translation")
    knowbase = LazyTool("_knowbase", AsyncKnowledgeBaseResource, "Knowledge base")
    banner = LazyTool("_banner", AsyncBannerResource, "Banner generator")
    crawler = LazyTool("_crawler", AsyncCrawlerResource, "Web scraping")
    email = LazyTool("_email", AsyncEmailResource, "Email gateway")
    telegram = LazyTool("_telegram", AsyncTelegramResource, "Telegram bots + MTProto")

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        llm_url: Optional[str] = None,
        api_url: Optional[str] = None,
        audio_url: Optional[str] = None,
        dev: bool = False,
        use_self_hosted: bool = True,
        openrouter_api_key: Optional[str] = None,
        provider: Optional[str] = None,
        timeout: float = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        **kwargs,
    ):
        """Initialize async SDKRouter. Same args as SDKRouter."""
        resolved_key, openai_base_url, _, _, config = resolve_init_params(
            api_key=api_key, llm_url=llm_url, api_url=api_url, audio_url=audio_url,
            dev=dev, use_self_hosted=use_self_hosted,
            openrouter_api_key=openrouter_api_key, provider=provider,
            timeout=timeout, max_retries=max_retries,
        )

        kwargs.pop("base_url", None)
        super().__init__(
            api_key=resolved_key,
            base_url=openai_base_url,
            timeout=timeout,
            max_retries=max_retries,
            **kwargs,
        )

        self._provider = provider
        self._sdk_config = config

    @property
    def embeddings(self) -> AsyncEmbeddingsResource:  # type: ignore[override]
        if not hasattr(self, "_embeddings") or self._embeddings is None:
            self._embeddings = AsyncEmbeddingsResource(self._sdk_config)
            self._embeddings._provider = self._provider
        return self._embeddings

    @property
    def audio(self) -> AsyncAudioResource:  # type: ignore[override]
        if not hasattr(self, "_audio") or self._audio is None:
            self._audio = AsyncAudioResource(self._sdk_config)
        return self._audio

    @property
    def config(self) -> SDKConfig:
        return self._sdk_config

    async def parse(
        self,
        *,
        model: str,
        messages: list[dict[str, Any]],
        response_format: Type[ResponseFormatT],
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_p: Optional[float] = None,
        frequency_penalty: Optional[float] = None,
        presence_penalty: Optional[float] = None,
        stop: Optional[list[str]] = None,
        seed: Optional[int] = None,
        **kwargs: Any,
    ) -> ParsedChatCompletion[ResponseFormatT]:
        """Async chat completion with structured output."""
        format_param = type_to_response_format(response_format)
        params = _build_parse_params(
            model, messages, format_param,
            temperature, max_tokens, top_p,
            frequency_penalty, presence_penalty, stop, seed,
            **kwargs,
        )
        merge_provider_extra_body(self._provider, params)
        raw = await self.chat.completions.create(**params)
        return parse_completion(raw, response_format)



"""Payments customers domain — sync and async mixins.

Customer endpoints use JWT auth — the customer is identified by the auth token,
not by customer_id parameter. These are for the customer-facing pay app.
"""

from __future__ import annotations

from typing import Any

from ...exceptions import api_error_handler, async_api_error_handler
from ..._api.generated.payments.payments__api__payments.models import (
    CustomerBalanceResponse,
    PaginatedCustomerInvoiceDetailList,
    PaginatedCustomerTransactionResponseList,
    PaginatedCustomerWithdrawalResponseList,
)


class _CustomersMixin:
    """Sync customer methods (JWT-authenticated)."""

    _api: Any

    @api_error_handler
    def customer_balance(self) -> CustomerBalanceResponse:
        """Get authenticated customer's balance."""
        return self._api.customer_balance_retrieve()

    @api_error_handler
    def customer_invoices(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerInvoiceDetailList:
        """List authenticated customer's invoices (payment history)."""
        return self._api.customer_invoices_list(page=page, page_size=page_size)

    @api_error_handler
    def customer_transactions(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerTransactionResponseList:
        """List authenticated customer's transactions."""
        return self._api.customer_transactions_list(page=page, page_size=page_size)

    @api_error_handler
    def customer_withdrawals(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerWithdrawalResponseList:
        """List authenticated customer's withdrawal requests."""
        return self._api.customer_withdrawals_list(page=page, page_size=page_size)


class _AsyncCustomersMixin:
    """Async customer methods (JWT-authenticated)."""

    _api: Any

    @async_api_error_handler
    async def customer_balance(self) -> CustomerBalanceResponse:
        """Get authenticated customer's balance."""
        return await self._api.customer_balance_retrieve()

    @async_api_error_handler
    async def customer_invoices(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerInvoiceDetailList:
        """List authenticated customer's invoices."""
        return await self._api.customer_invoices_list(page=page, page_size=page_size)

    @async_api_error_handler
    async def customer_transactions(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerTransactionResponseList:
        """List authenticated customer's transactions."""
        return await self._api.customer_transactions_list(page=page, page_size=page_size)

    @async_api_error_handler
    async def customer_withdrawals(
        self,
        *,
        page: int = 1,
        page_size: int = 20,
    ) -> PaginatedCustomerWithdrawalResponseList:
        """List authenticated customer's withdrawal requests."""
        return await self._api.customer_withdrawals_list(page=page, page_size=page_size)
